<?php
//
// Deployment Management
// Copyright (c) 2020 Nagios Enterprises, LLC. All rights reserved.
//
// IMPORTANT: SOURCE CODE MAY NOT BE DISCLOSED OR DISTRIBUTED IN UNENCODED FORM
//            WITHOUT EXPLICIT PERMISSION FROM NAGIOS ENTERPRISES, AS IT CONTAINS
//            CODE LICENSED FROM THIRD PARTIES.
//

require_once(dirname(__FILE__) . '/../../includes/common.inc.php');
require_once(dirname(__FILE__) . '/includes/utils-deployment.inc.php');

// Initialization stuff
pre_init();
init_session();

// Grab GET or POST variables, check prereqs and authentication
grab_request_vars();
check_prereqs();
check_authentication(false);


// Check if people have access to deployment
if (!user_has_permission('autodeploy_access')) {
    echo _('Access denied. You do not have permision to access this component.');
    die();
}


route_request();


function route_request()
{
    $cmd = grab_request_var("cmd", "");

    switch ($cmd) {

        case 'add':
            print add_agent();
            break;

        case 'edit':
            print edit_agent();
            break;

        case 'getagents':
            print json_encode(deploy_get_agents());
            break;

        case 'delete':
            $id = grab_request_var("id", 0);
            if (!empty($id)) {
                $x = deploy_delete_agent($id);
                if ($x) {
                    flash_message(_('Agent removed from the deployed agents list.'));
                } else {
                    flash_message(_('Could not remove specified agent.'), FLASH_MSG_ERROR);
                }
            }
            header("Location: manage.php");
            break;

        default:
            show_page();
            break;

    }   
}


function add_agent()
{
    $hostname = grab_request_var('hostname', '');
    $os = grab_request_var('os', 'linux');
    $agent = grab_request_var('agent', 'ncpa');
    $token =  grab_request_var('token', '');

    // Must fill out form
    if (empty($hostname) || empty($token)) {
        return json_encode(array('success' => 0, 'msg' => _('Must fill out the entire form.')));
    }

    // Sanatize agent os
    if (!in_array($os, array('linux', 'windows', 'aix', 'macosx', 'solaris'))) {
        $os = "linux";
    }
    if (!in_array($agent, array('ncpa'))) {
        $agent = "ncpa";
    }

    // Verify agent information
    $data = deploy_get_ncpa_api_data('system/agent_version', 'https://'.$hostname.':5693/api', $token);
    if (empty($data)) {
        return json_encode(array('success' => 0, 'msg' => sprintf(_('Could not connect to <b>%s</b>'), "https://$hostname:5693")));
    } else if (isset($data['error'])) {
        return json_encode(array('success' => 0, 'msg' => sprintf(_('Could not connect to <b>%s</b> with token given.'), "https://$hostname:5693")));
    }

    $job = array('ncpa_token' => encrypt_data($token),
                 'creator_id' => $_SESSION['user_id'],
                 'os' => $os);
    $x = deploy_add_deployed_host($hostname, $job);

    if ($x) {
        return json_encode(array('success' => 1));
    } else {
        return json_encode(array('success' => 0, 'msg' => _('Could not add the agent.')));
    }
}


function edit_agent()
{
    $id = grab_request_var('id', '');
    $hostname = grab_request_var('hostname', '');
    $os = grab_request_var('os', 'linux');
    $agent = grab_request_var('agent', 'ncpa');
    $token =  grab_request_var('token', '');

    // Must fill out form
    if (empty($id) || empty($hostname)) {
        return json_encode(array('success' => 0, 'msg' => _('Must fill out the entire form.')));
    }

    // Sanatize agent os
    if (!in_array($os, array('linux', 'windows', 'aix', 'macosx', 'solaris'))) {
        $os = "linux";
    }
    if (!in_array($agent, array('ncpa'))) {
        $agent = "ncpa";
    }

    // Verify agent information
    $test_token = $token;
    if (empty($token))  {
        $agent = deploy_get_agent($id);
        $data = json_decode(base64_decode($agent['metadata']), true);
        $test_token = decrypt_data($data['token']);
    }
    $data = deploy_get_ncpa_api_data('system/agent_version', 'https://'.$hostname.':5693/api', $test_token);
    if (empty($data)) {
        return json_encode(array('success' => 0, 'msg' => sprintf(_('Could not connect to <b>%s</b>'), "https://$hostname:5693")));
    } else if (isset($data['error'])) {
        return json_encode(array('success' => 0, 'msg' => sprintf(_('Could not connect to <b>%s</b> with token given.'), "https://$hostname:5693")));
    }

    $opts = array('os' => $os);
    if (!empty($token)) {
        $opts['ncpa_token'] = encrypt_data($token);
    }

    $x = deploy_edit_deployed_host($id, $hostname, $opts);

    if ($x) {
        return json_encode(array('success' => 1));
    } else {
        return json_encode(array('success' => 0, 'msg' => _('Could not edit the agent.')));
    }
}


function show_page()
{
    $refresh = intval(get_user_meta(0, 'deploy_manage_page_refresh', 30));
    if ($refresh < 1) {
        $refresh = 1;
    }

    $format = '';
    if (is_null($format = get_user_meta(0, 'date_format'))) { $format = get_option('default_date_format'); }

    $url = get_base_url()."config/deployment/manage.php";

    $page = intval(grab_request_var('page', 0));
    $records = intval(grab_request_var("records", get_user_meta(0, 'deploy_recordlimit', 10)));
    $total_records = deploy_get_agents(true);
    $pager_results = get_table_pager_info("", $total_records, $page, $records);

    do_page_start(array('page_title' => _('Manage Deployed Agents')), true, true);
?>

<script type="text/javascript">
// Selected ids (to preserve when ajax calls happen)s
var PAGE_TO_ID = 0;
var sem = 0;

var record_limit = <?php echo $pager_results['records_per_page']; ?>;
var current_page = <?php echo $pager_results['current_page']; ?>;
var total_records = <?php echo $pager_results['total_records']; ?>;
var total_pages = <?php echo $pager_results['total_pages']; ?>;

function get_agents(force)
{
    var data = { records: $('.num-records').val(), page: current_page };

    data.cmd = 'getagents';
    $.ajax({
        type: "GET",
        url: '<?php echo $url; ?>',
        data: data,
        dataType: 'json',
        success: function(agents) {
            obj.agents = agents;
        }
    });
}

$(document).ready(function() {

    $('.next-page').click(function() {
        current_page++;
        update_pages();
    });

    $('.first-page').click(function() {
        current_page = 1;
        update_pages();
    });

    $('.last-page').click(function() {
        current_page = total_pages;
        update_pages();
    });

    $('.previous-page').click(function() {
        if (current_page > 1) {
            current_page--;
        }
        update_pages();
    });

    $('.num-records').change(function() {
        $('.num-records').val($(this).val());
        record_limit = $(this).val();
        current_page = 1;
        update_pages();
    });

    // Special things for Manage Deployment page

    $('#add-agent-btn').click(function() {
        whiteout();
        $('#add-agent-modal').center().css('top', 120).show();
    });

    $('#add-agent-modal').on('click', '.btn-cancel', function() {
        clear_whiteout();
        $('#add-agent-modal').hide();
    });

    $('#add-agent-modal').on('click', '.btn-confirm', function() {
        var btn = $(this).button('loading');
        var form_data = {
            cmd: "add",
            hostname: $('#add-form input[name=hostname]').val(),
            os: $('#add-form select[name=os]').val(),
            agent: $('#add-form select[name=agent]').val(),
            token: $('#add-form input[name=token]').val()
        }

        $.post('manage.php', form_data, function(data) {
            if (data.success) {
                clear_whiteout();
                $('#add-agent-modal').hide();
                get_agents();
            } else {
                $('#add-form .error').html(data.msg);
                $('#add-form .error').show();
                $('#add-agent-modal').center().css('top', 120);
            }
            btn.button('reset');
        }, 'json');
    });

    $('#edit-agent-modal').on('click', '.btn-cancel', function() {
        clear_whiteout();
        $('#edit-agent-modal').hide();
    });

    $('#edit-agent-modal').on('click', '.btn-confirm', function() {
        var btn = $(this).button('loading');
        var form_data = {
            cmd: "edit",
            id: $('#edit-form input[name=id]').val(),
            hostname: $('#edit-form input[name=hostname]').val(),
            os: $('#edit-form select[name=os]').val(),
            agent: $('#edit-form select[name=agent]').val(),
            token: $('#edit-form input[name=token]').val()
        }

        $.post('manage.php', form_data, function(data) {
            if (data.success) {
                clear_whiteout();
                $('#edit-agent-modal').hide();
                get_agents();
                $('#edit-form input[name=token]').val('')
            } else {
                $('#edit-form .error').html(data.msg);
                $('#edit-form .error').show();
                $('#edit-agent-modal').center().css('top', 120);
            }
            btn.button('reset');
        }, 'json');
    });

});

function update_pages(reload) {
    if (reload === undefined) { reload = true; }

    var curr_page_html = '<?php echo _("Page"); ?> ' + current_page + ' <?php echo _("of"); ?> ' + total_pages;
    $('.ajax-pagination span').html(curr_page_html);

    // Do hide/show options
    if (current_page > 1) {
        $('.previous-page').attr('disabled', false);
    } else if (current_page == 1) {
        $('.previous-page').attr('disabled', true);
    }

    if (current_page == total_pages) {
        $('.next-page').attr('disabled', true);
    } else {
        $('.next-page').attr('disabled', false);
    }

    if (reload) {
        get_downtimes(true);
    }
}
</script>

<h1><?php echo _('Manage Deployed Agents'); ?></h1>

<p style="padding: 0; margin: 10px 0 20px 0;"><?php echo _('Successfully deployed agents will be listed below and can be updated by clicking the update button. You can also remove the agent from the list. Removing agents from the list will not remove the agent from the host system it is installed on.'); ?></p>

<p style="margin-bottom: 10px;">
    <a href="index.php" class="btn btn-sm btn-primary"><i class="fa fa-play l"></i> <?php echo _('Deploy Agent'); ?></a> &nbsp;
    <button type="button" id="add-agent-btn" class="btn btn-sm btn-default"><i class="fa fa-plus l"></i> <?php echo _('Add Agent'); ?></button>
</p>

<div id="agents">
    <table class="table table-condensed tabel-striped table-bordered table-no-margin table-hover">
        <thead>
            <tr>
                <th><?php echo _('IP Address/Hostname'); ?></th>
                <th style="width: 120px;"><?php echo _('Agent Status'); ?></th>
                <th style="width: 70px;"><?php echo _('Agent'); ?></th>
                <th style="width: 100px;"><?php echo _('Agent Version'); ?></th>
                <th style="width: 100px;"><?php echo _('OS'); ?></th>
                <th><?php echo _('Last Status Check'); ?></th>
                <th><?php echo _('Last Updated'); ?></th>
                <th><?php echo _('Creator'); ?></th>
                <th><?php echo _('Deployed / Added On'); ?></th>
                <th style="width: 120px; text-align: center;"><?php echo _('Actions'); ?></th>
            </tr>
        </thead>
        <tbody>
            <tr v-for="agent in agents" v-bind:key="agent.deploy_id">
                <td>{{ agent.hostname }}</td>
                <td v-bind:class="{ hostup: agent.available == 1, hostdown: agent.available == 0 }">{{ get_status(agent.available) }}</td>
                <td>NCPA</td>
                <td>{{ agent.version }}</td>
                <td>{{ agent.metadata.os_name + " " + agent.metadata.os_version.split('.')[0] }}</td>
                <td>{{ get_datetime(agent.last_status_check) }}</td>
                <td>{{ get_datetime(agent.last_updated_date) }}</td>
                <td>{{ agent.name }}</td>
                <td>{{ get_datetime(agent.deployed_date) }}</td>
                <td style="text-align: center;">
                    <a v-on:click="do_action" v-bind:href="'../monitoringwizard.php?wizard=ncpa&nextstep=3&nsp=<?php echo get_nagios_session_protector_id(); ?>&deploy_id='+agent.deploy_id" class="tt-bind" title="<?php echo _('Run Wizard'); ?>"><img src="<?php echo theme_image('wizardwand.png'); ?>"></a>&nbsp;
                    <a v-on:click="do_edit(agent.deploy_id)" class="tt-bind" title="<?php echo _('Edit'); ?>"><img src="<?php echo theme_image('pencil.png'); ?>"></a>&nbsp;
                    <a v-bind:href="'manage.php?cmd=delete&id='+agent.deploy_id" class="tt-bind" title="<?php echo _('Remove'); ?>"><img src="<?php echo theme_image('cross.png'); ?>"></a>
                </td>
            </tr>
            <tr v-if="!agents.length">
                <td colspan="9"><?php echo _('No agents have been deployed or added yet.'); ?></td>
            </tr>
        </tbody>
    </table>
</div>

<!--
<div>
    <div class="fr">
        <div class="ajax-pagination" style="margin: 20px 0 0 0;">
            <button class="btn btn-xs btn-default first-page" title="<?php echo _('First Page'); ?>"><i class="fa fa-fast-backward"></i></button>
            <button class="btn btn-xs btn-default previous-page" title="<?php echo _('Previous Page'); ?>" disabled><i class="fa fa-chevron-left l"></i></button>
            <span style="margin: 0 10px;"><?php echo _('Page'); ?> <span class="pagenum">1 <?php echo _('of'); ?> <?php echo $pager_results['total_pages']; ?></span></span>
            <button class="btn btn-xs btn-default next-page" title="<?php echo _('Next Page'); ?>"><i class="fa fa-chevron-right r"></i></button>
            <button class="btn btn-xs btn-default last-page" title="<?php echo _('Last Page'); ?>"><i class="fa fa-fast-forward"></i></button>

            <select class="form-control condensed num-records">
                <option value="5"<?php if ($pager_results['records_per_page'] == 5) { echo ' selected'; } ?>>5 <?php echo _('Per Page'); ?></option>
                <option value="10"<?php if ($pager_results['records_per_page'] == 10) { echo ' selected'; } ?>>10 <?php echo _('Per Page'); ?></option>
                <option value="25"<?php if ($pager_results['records_per_page'] == 25) { echo ' selected'; } ?>>25 <?php echo _('Per Page'); ?></option>
                <option value="50"<?php if ($pager_results['records_per_page'] == 50) { echo ' selected'; } ?>>50 <?php echo _('Per Page'); ?></option>
                <option value="100"<?php if ($pager_results['records_per_page'] == 100) { echo ' selected'; } ?>>100 <?php echo _('Per Page'); ?></option>
                <option value="250"<?php if ($pager_results['records_per_page'] == 250) { echo ' selected'; } ?>>250 <?php echo _('Per Page'); ?></option>
                <option value="500"<?php if ($pager_results['records_per_page'] == 500) { echo ' selected'; } ?>>500 <?php echo _('Per Page'); ?></option>
                <option value="-1"<?php if ($pager_results['records_per_page'] == -1) { echo ' selected'; } ?>><?php echo _('No Limit'); ?></option>
            </select>

            <input type="text" class="form-control condensed jump-to">
            <button class="btn btn-xs btn-default tt-bind jump" title="<?php echo _('Jump to Page'); ?>"><i class="fa fa-chevron-circle-right fa-12"></i></button>
        </div>
    </div>
    <div class="clear"></div>
</div>
-->

<div id="add-agent-modal" class="xi-modal hide">
    <h2><?php echo _('Add Agent'); ?></h2>
    <p style="margin: 20px 0; padding: 0;"><?php echo _('Manually add an agent that has been deployed to your environment without using the Nagios XI deployment tool. Tokens are encrypted and used when running the wizard.'); ?></p>
    <div id="add-form" class="form-horizontal form-modern">

        <div class="alert alert-danger error hide"></div>

        <div class="form-group">
            <label class="control-label"><?php echo _('IP Address / Hostname'); ?></label>
            <div>
                <input type="text" class="form-control" name="hostname" value="" style="width: 240px;">
            </div>
        </div>

        <div class="form-group">
            <label class="control-label"><?php echo _('Operating System'); ?></label>
            <div>
                <select name="os" class="os form-control">
                    <option value="linux">Linux</option>
                    <option value="windows">Windows</option>
                    <option value="macosx">Mac OS X</option>
                    <option value="aix">AIX</option>
                    <option value="solaris">Solaris</option>
                </select>
            </div>
        </div>

        <div class="form-group">
            <label class="control-label"><?php echo _('Agent Software'); ?></label>
            <div>
                <select name="agent" class="agent form-control">
                    <option value="ncpa">NCPA</option>
                </select>
            </div>
        </div>

        <div class="form-group">
            <label class="control-label"><?php echo _('NCPA Token'); ?></label>
            <div>
                <input type="text" class="form-control" name="token" value="" style="width: 200px;">
            </div>
        </div>

    </div>
    <div style="padding-top: 20px;">
        <button type="button" data-loading-text="<?php echo _('Adding...'); ?>" class="btn-confirm btn btn-sm btn-primary"><?php echo _('Add'); ?></button>
        <button type="button" class="btn-cancel btn btn-sm btn-default"><?php echo _('Cancel'); ?></button>
    </div>
</div>

<div id="edit-agent-modal" class="xi-modal hide">
    <h2><?php echo _('Edit Agent'); ?></h2>
    <p style="margin: 20px 0; padding: 0;"><?php echo _('Tokens are encrypted and used when running the wizard.'); ?></p>
    <div id="edit-form" class="form-horizontal form-modern">
        <input type="hidden" name="id" value="">

        <div class="alert alert-danger error hide"></div>

        <div class="form-group">
            <label class="control-label"><?php echo _('IP Address / Hostname'); ?></label>
            <div>
                <input type="text" class="form-control" name="hostname" value="" style="width: 240px;">
            </div>
        </div>

        <div class="form-group">
            <label class="control-label"><?php echo _('Operating System'); ?></label>
            <div>
                <select name="os" class="os form-control">
                    <option value="linux">Linux</option>
                    <option value="windows">Windows</option>
                    <option value="macosx">Mac OS X</option>
                    <option value="aix">AIX</option>
                    <option value="solaris">Solaris</option>
                </select>
            </div>
        </div>

        <div class="form-group">
            <label class="control-label"><?php echo _('Agent Software'); ?></label>
            <div>
                <select name="agent" class="agent form-control">
                    <option value="ncpa">NCPA</option>
                </select>
            </div>
        </div>

        <div class="form-group">
            <label class="control-label"><?php echo _('NCPA Token'); ?></label>
            <div>
                <input type="text" class="form-control" name="token" value="" style="width: 200px;">
            </div>
        </div>

    </div>
    <div style="padding-top: 20px;">
        <button type="button" data-loading-text="<?php echo _('Saving...'); ?>" class="btn-confirm btn btn-sm btn-primary"><?php echo _('Save'); ?></button>
        <button type="button" class="btn-cancel btn btn-sm btn-default"><?php echo _('Cancel'); ?></button>
    </div>
</div>

<script>
// Vue creation/data
var obj = {
    agents: <?php echo json_encode(deploy_get_agents()); ?>
};

var main = new Vue({
    el: '#agents',
    data: obj,
    methods: {
        get_status: function(available) {
            if (available == 1) {
                return '<?php echo _("Available"); ?>';
            }
            return '<?php echo _("Not available"); ?>';
        },
        get_datetime: function(time) {
            var format = parseInt('<?php echo $format; ?>');
            var formatted = '';
            if (time === null) { return ""; }
            var date = new Date(time);

            if (format == 2) {
                // MM/DD/YYYY HH:MM:SS
                formatted += ('0'+(date.getMonth()+1)).slice(-2)+'/';
                formatted += ('0'+date.getDate()).slice(-2)+'/';
                formatted += date.getFullYear()+' ';
                
            } else if (format == 3) {
                // DD/MM/YYYY HH:MM:SS
                formatted += ('0'+date.getDate()).slice(-2)+'/';
                formatted += ('0'+(date.getMonth()+1)).slice(-2)+'/';
                formatted += date.getFullYear()+' ';
            } else {
                // YYYY-MM-DD HH:MM:SS
                formatted += date.getFullYear()+'-';
                formatted += ('0'+(date.getMonth()+1)).slice(-2)+'-';
                formatted += ('0'+date.getDate()).slice(-2)+' ';
            }

            formatted += ('0'+date.getHours()).slice(-2)+':';
            formatted += ('0'+date.getMinutes()).slice(-2)+':';
            formatted += ('0'+date.getSeconds()).slice(-2);

            return formatted;
        },
        do_action: function() {
            whiteout();
            show_throbber();
        },
        do_edit: function(id) {
            whiteout();
            var modal = $('#edit-agent-modal');
            modal.css('min-width', 475).center().css('top', 120).show();
            modal.find('.error').hide();

            // Get the agent from the list
            var agent = '';
            obj.agents.forEach(function(o, i) {
                if (o.deploy_id == id) {
                    agent = o;
                }
            });

            // populate the agent info
            modal.find('input[name=id]').val(agent.deploy_id);
            modal.find('input[name=hostname]').val(agent.hostname);
            modal.find('select[name=os]').val(agent.os);
            modal.find('input[name=token]').val('');
        }
    }
});
</script>

<?php
    do_page_end(true);
}
